@@ -9,6 +9,7 @@ use Carp         ();
 use Text::Xslate::Parser;
 use Text::Xslate::Util qw(
     $DEBUG
+    literal_to_value
     value_to_literal
     is_int any_in
     make_error
@@ -28,9 +29,6 @@ use constant {
     _OP_FILE    => 3,
     _OP_LABEL   => 4,
     _OP_COMMENT => 5,
-
-    _FOR_LOOP   => 1,
-    _WHILE_LOOP => 2,
 };
 
 
@@ -92,37 +90,16 @@ my %unary = (
     'max_index' => 'max_index', # for loop context vars
 );
 
-my %goto_family = map { $_ => undef } qw(
-    for_iter
-    and
-    dand
-    or
-    dor
-    goto
-);
-
 my %builtin = (
-    'html_escape'  => ['builtin_html_escape',
-                        \&Text::Xslate::Util::html_escape],
-    'uri_escape'   => ['builtin_uri_escape',
-                        \&Text::Xslate::Util::uri_escape],
-    'mark_raw'     => ['builtin_mark_raw',
-                        \&Text::Xslate::Util::mark_raw],
-    'unmark_raw'   => ['builtin_unmark_raw',
-                        \&Text::Xslate::Util::unmark_raw],
+    'raw'          => 'builtin_mark_raw',
+    'html'         => 'builtin_html_escape',
 
-    'raw'          => ['builtin_mark_raw',
-                        \&Text::Xslate::Util::mark_raw],
+    'mark_raw'     => 'builtin_mark_raw',
+    'unmark_raw'   => 'builtin_unmark_raw',
 
-    'html'         => ['builtin_html_escape',
-                        \&Text::Xslate::Util::html_escape],
-    'uri'          => ['builtin_uri_escape',
-                        \&Text::Xslate::Util::uri_escape],
-
-    'is_array_ref' => ['builtin_is_array_ref',
-                        \&Text::Xslate::Util::is_array_ref],
-    'is_hash_ref'  => ['builtin_is_hash_ref',
-                        \&Text::Xslate::Util::is_hash_ref],
+    'uri'          => 'builtin_uri',
+    'is_array_ref' => 'builtin_is_array_ref',
+    'is_hash_ref'  => 'builtin_is_hash_ref',
 );
 
 has lvar_id => ( # local varialbe id
@@ -156,7 +133,6 @@ has macro_table => (
 
 has engine => ( # Xslate engine
     is       => 'ro',
-    isa      => 'Object',
     required => 0,
     weak_ref => 1,
 );
@@ -197,6 +173,12 @@ has parser => (
     init_arg => undef,
 );
 
+has file => (
+    is  => 'rw',
+
+    init_arg => undef,
+);
+
 has input_layer => (
     is      => 'ro',
     default => ':utf8',
@@ -226,7 +208,7 @@ has cascade => (
     init_arg => undef,
 );
 
-has [qw(header footer macro)] => (
+has [qw(header footer)] => (
     is  => 'rw',
     isa => 'ArrayRef',
 );
@@ -237,19 +219,6 @@ has current_file => (
     init_arg => undef,
 );
 
-has file => (
-    is  => 'rw',
-
-    init_arg => undef,
-);
-
-has overridden_builtin => (
-    is  => 'ro',
-    isa => 'HashRef',
-
-    default => sub { +{} },
-);
-
 sub lvar_use {
     my($self, $n) = @_;
 
@@ -270,32 +239,19 @@ sub compile {
     local $self->{lvar_id     } = 0;
     local $self->{lvar}         = {};
     local $self->{const}        = [];
-    local $self->{in_loop}      = 0;
     local $self->{dependencies} = [];
     local $self->{cascade};
     local $self->{header}       = $self->{header};
     local $self->{footer}       = $self->{footer};
-    local $self->{macro}        = $self->{macro};
     local $self->{current_file} = '<string>'; # for opinfo
     local $self->{file}         = $args{file} || \$input;
 
-    if(my $engine = $self->engine) {
-        my $ob = $self->overridden_builtin;
-        Internals::SvREADONLY($ob, 0);
-        foreach my $name(keys %builtin) {
-            my $f = $engine->{function}{$name};
-            $ob->{$name} = ( $builtin{$name}[1] != $f ) + 0;
-        }
-        Internals::SvREADONLY($ob, 1);
-    }
-
     my $parser = $self->parser;
 
     my $header = delete $self->{header};
     my $footer = delete $self->{footer};
-    my $macro = delete $self->{macro};
 
-    if(!$args{omit_augment}) {
+    if(!$args{from_include}) {
         if($header) {
             substr $input, 0, 0, $self->_cat_files($header);
         }
@@ -303,11 +259,6 @@ sub compile {
             $input .= $self->_cat_files($footer);
         }
     }
-    if($macro) {
-        if(!grep { $_ eq $self->current_file } @$macro) {
-            substr $input, 0, 0, $self->_cat_files($macro);
-        }
-    }
 
     my @code; # main code
     {
@@ -386,7 +337,7 @@ sub _cat_files {
     my $s = '';
     foreach my $file(@{$files}) {
         my $fullpath = $engine->find_file($file)->{fullpath};
-        $s .= $engine->slurp_template( $self->input_layer, $fullpath );
+        $s .= $engine->slurp( $fullpath );
         $self->requires($fullpath);
     }
     return $s;
@@ -402,12 +353,12 @@ sub compile_ast {
 
     my @code;
     foreach my $node(ref($ast) eq 'ARRAY' ? @{$ast} : $ast) {
-        blessed($node) or Carp::confess("[BUG] Not a node object: " . p($node));
+        blessed($node) or Carp::confess("Oops: Not a node object: " . p($node));
 
         printf STDERR "%s"."generate %s (%s)\n", "." x $_lv, $node->arity, $node->id if _DUMP_GEN;
 
         my $generator = $self->can('_generate_' . $node->arity)
-            || Carp::confess("[BUG] Unexpected node:  " . p($node));
+            || Carp::confess("Oops: Unexpected node:  " . p($node));
 
         push @code, $self->$generator($node);
     }
@@ -417,7 +368,7 @@ sub compile_ast {
 
 sub _process_cascade {
     my($self, $cascade, $args, $main_code) = @_;
-    printf STDERR "# cascade %s %s", $self->file, $cascade->dump if _DUMP_CAS;
+    printf STDERR "cascade %s\n", $self->file, $cascade->dump if _DUMP_CAS;
     my $engine = $self->engine
         || $self->_error("Cannot cascade templates without Xslate engine", $cascade);
 
@@ -495,7 +446,7 @@ sub _process_cascade {
         foreach my $c(@{$main_code}) {
             if($c->[_OP_NAME] eq 'print_raw_s'
                     && $c->[_OP_ARG] =~ m{ [^ \t\r\n] }xms) {
-                Carp::carp("Xslate: Useless use of text '$c->[1]'");
+                Carp::carp("Xslate: Uselses use of text '$c->[1]'");
             }
         }
         @{$main_code} = @{$base_code};
@@ -507,7 +458,7 @@ sub _process_cascade {
 
 sub _process_cascade_file {
     my($self, $file, $base_code) = @_;
-    printf STDERR "# cascade file %s\n", p($file) if _DUMP_CAS;
+    printf STDERR "cascade file %s\n", Mouse::Util::dump($file) if _DUMP_CAS;
     my $mtable = $self->macro_table;
 
     for(my $i = 0; $i < @{$base_code}; $i++) {
@@ -518,20 +469,13 @@ sub _process_cascade_file {
 
         # macro
         my $name = $c->[_OP_ARG];
-        $name =~ s/\@.+$//;
-        printf STDERR "# macro %s\n", $name if _DUMP_CAS;
+        #warn "# macro ", $name, "\n";
 
         if(exists $mtable->{$name}) {
-            my $m = $mtable->{$name};
-            if(ref($m) ne 'HASH') {
-                $self->_error('[BUG] Unexpected macro structure: '
-                    . p($m) );
-            }
-
             $self->_error(
                 "Redefinition of macro/block $name in " . $file
                 . " (you must use block modifiers to override macros/blocks)",
-                $m->{line}
+                $mtable->{$name}{line}
             );
         }
 
@@ -629,21 +573,18 @@ sub _generate_operator {
     $self->_error("Invalid expression", $node);
 }
 
-sub _can_optimize_print {
+sub _can_print_optimize {
     my($self, $name, $node) = @_;
 
     return 0 if !$OPTIMIZE;
     return 0 if !($name eq 'print' or $name eq 'print_raw');
 
-    my $maybe_name = $node->first;
     return $node->arity eq 'call'
-        && $maybe_name->arity eq 'name'
+        && $node->first->arity eq 'name'
         && @{$node->second} == 1 # args of the filter
-        && any_in($maybe_name->id, qw(raw mark_raw html))
-        && !$self->overridden_builtin->{$maybe_name->id};
+        && any_in($node->first->id, qw(raw mark_raw html));
 }
 
-# also deal with smart escaping
 sub _generate_print {
     my($self, $node) = @_;
 
@@ -655,35 +596,18 @@ sub _generate_print {
     }
 
     foreach my $arg(@{ $node->first }){
-        if( $proc eq 'print' && $self->overridden_builtin->{html_escape} ) {
-            # default behaviour of print() is overridden
-            push @code,
-                $self->opcode('pushmark'),
-                $self->compile_ast($arg),
-                $self->opcode('push'),
-                $self->opcode('fetch_symbol' => 'html_escape'),
-                $self->opcode('funcall'),
-                $self->opcode('print_raw');
+        if(exists $Text::Xslate::OPS{$proc . '_s'} && $arg->arity eq 'literal'){
+            push @code, $self->opcode( $proc . '_s' => $arg->value );
         }
-        elsif(exists $Text::Xslate::OPS{$proc . '_s'}
-                && $arg->arity eq 'literal'){
-            push @code,
-                $self->opcode( $proc . '_s' => $arg->value,
-                               line         => $arg->line );
-        }
-        elsif($self->_can_optimize_print($proc, $arg)){
+        elsif($self->_can_print_optimize($proc, $arg)){
             my $filter      = $arg->first;
             my $filter_name = $filter->id;
-            my $command = $builtin{ $filter_name }[0] eq 'builtin_mark_raw'
+            my $command = $builtin{ $filter_name } eq 'builtin_mark_raw'
                 ? 'print_raw'  # mark_raw, raw
                 : 'print';     # html
-
             push @code,
                 $self->compile_ast($arg->second->[0]),
-                $self->opcode(
-                    $command => undef,
-                    symbol   => $filter );
-
+                $self->opcode( $command => undef, symbol => $filter );
         }
         else {
             push @code,
@@ -708,7 +632,6 @@ sub _generate_include {
             : $self->compile_ast($file) ),
         $self->opcode( $node->id => undef, line => $node->line ),
     );
-
     if(defined(my $vars = $node->second)) {
         @code = ($self->opcode('enter'),
             $self->_localize_vars($vars),
@@ -741,13 +664,12 @@ sub _generate_cascade {
     return;
 }
 
-# XXX: need more consideration
-sub _compile_loop_block {
+sub _compile_block {
     my($self, $block) = @_;
     my @block_code = $self->compile_ast($block);
 
     foreach my $op(@block_code) {
-        if(any_in( $op->[_OP_NAME], qw(pushmark loop_control))) {
+        if($op->[_OP_NAME] eq 'pushmark') {
             # pushmark ... funcall (or something) may create mortal SVs
             # so surround the block with ENTER and LEAVE
             unshift @block_code, $self->opcode('enter');
@@ -756,18 +678,6 @@ sub _compile_loop_block {
         }
     }
 
-    foreach my $i(1 .. (@block_code-1)) {
-        my $op = $block_code[$i];
-        if($op->[_OP_NAME] eq 'loop_control') {
-            my $type = $op->[_OP_ARG];
-            $op->[_OP_NAME] = 'goto';
-
-            $op->[_OP_ARG] = (@block_code - $i);
-
-            $op->[_OP_ARG] += 1 if $type eq 'last';
-        }
-    }
-
     return @block_code;
 }
 
@@ -780,9 +690,8 @@ sub _generate_for {
     if(@{$vars} != 1) {
         $self->_error("A for-loop requires single variable for each item", $node);
     }
-    local $self->{lvar}  = { %{$self->lvar} };  # new scope
-    local $self->{const} = [ @{$self->const} ]; # new scope
-    local $self->{in_loop} = _FOR_LOOP;
+    local $self->{lvar}  = { %{$self->lvar} }; # new scope
+    local $self->{const} = [];                 # new scope
 
     my @code = $self->compile_ast($expr);
 
@@ -791,14 +700,13 @@ sub _generate_for {
     my $lvar_name = $iter_var->id;
 
     $self->lvar->{$lvar_name} = $lvar_id;
-    $self->lvar->{'($_)'}     = $lvar_id;
 
     push @code, $self->opcode( for_start => $lvar_id, symbol => $iter_var );
 
     # a for statement uses three local variables (container, iterator, and item)
     local $self->{lvar_id} = $self->lvar_use(3);
 
-    my @block_code = $self->_compile_loop_block($block);
+    my @block_code = $self->_compile_block($block);
     push @code,
         $self->opcode( literal_i => $lvar_id, symbol => $iter_var ),
         $self->opcode( for_iter  => scalar(@block_code) + 2 ),
@@ -808,26 +716,6 @@ sub _generate_for {
     return @code;
 }
 
-sub _generate_for_else {
-    my($self, $node) = @_;
-
-    my $for_block  = $node->first;
-    my $else_block = $node->second;
-
-    my @code = (
-        $self->compile_ast($for_block),
-    );
-
-    # 'for' block sets __a with true if the loop count > 0
-    my @else = $self->compile_ast($else_block);
-    push @code, (
-        $self->opcode( or => scalar(@else) + 1, comment => 'for-else' ),
-        @else,
-    );
-
-    return @code;
-}
-
 sub _generate_while {
     my($self, $node) = @_;
     my $expr  = $node->first;
@@ -840,10 +728,8 @@ sub _generate_while {
 
     (my $cond_op, undef, $expr) = $self->_prepare_cond_expr($expr);
 
-    # TODO: combine all the loop contexts into single one
     local $self->{lvar}  = { %{$self->lvar}  }; # new scope
     local $self->{const} = [ @{$self->const} ]; # new scope
-    local $self->{in_loop} = _WHILE_LOOP;
 
     my @code = $self->compile_ast($expr);
 
@@ -858,7 +744,7 @@ sub _generate_while {
     }
 
     local $self->{lvar_id} = $self->lvar_use(scalar @{$vars});
-    my @block_code = $self->_compile_loop_block($block);
+    my @block_code = $self->_compile_block($block);
     return @code,
         $self->opcode( $cond_op => scalar(@block_code) + 2, symbol => $node ),
         @block_code,
@@ -867,38 +753,6 @@ sub _generate_while {
     return @code;
 }
 
-sub _generate_loop_control {
-    my($self, $node) = @_;
-    my $type = $node->id;
-
-    any_in($type, qw(last next))
-        or $self->_error("[BUG] Unknown loop control statement '$type'");
-
-    if(not $self->{in_loop}) {
-        $self->_error("Use of loop control statement ($type) outside of loops");
-    }
-
-    my @cleanup;
-    if( $self->{in_loop} == _FOR_LOOP && $type eq 'last' ) {
-        my $lvar_id = $self->lvar->{'($_)'};
-        defined($lvar_id)
-            or $self->_error('[BUG] Undefined loop iterator');
-
-        @cleanup = (
-            $self->opcode( 'nil', undef,
-                comment => 'to clean the loop context' ),
-            $self->opcode( save_to_lvar => $lvar_id + 0), # item
-            $self->opcode( save_to_lvar => $lvar_id + 1), # iterator
-            $self->opcode( save_to_lvar => $lvar_id + 2), # body
-            $self->opcode( literal_i    => 1 ), # for 'for-else'
-        );
-    }
-
-    return $self->opcode('leave'),
-           @cleanup,
-           $self->opcode('loop_control' => $type, comment => $type);
-}
-
 sub _generate_proc { # definition of macro, block, before, around, after
     my($self, $node) = @_;
     my $type   = $node->id;
@@ -1084,10 +938,10 @@ sub _generate_variable {
     }
 }
 
-sub _generate_super {
+sub _generate_marker {
     my($self, $node) = @_;
 
-    return return $self->opcode( super => undef, symbol => $node );
+    return return $self->opcode( $node->id => undef, symbol => $node );
 }
 
 sub _generate_literal {
@@ -1100,11 +954,6 @@ sub _generate_nil {
     return $self->opcode('nil');
 }
 
-sub _generate_vars {
-    my($self) = @_;
-    return $self->opcode('vars');
-}
-
 sub _generate_composer {
     my($self, $node) = @_;
 
@@ -1123,12 +972,11 @@ sub _generate_unary {
 
     my $id = $node->id;
     if(exists $unary{$id}) {
-        my @operand = $self->compile_ast($node->first);
         my @code = (
-            @operand,
+            $self->compile_ast($node->first),
             $self->opcode( $unary{$id} )
         );
-        if( $OPTIMIZE and $self->_code_is_literal(@operand) ) {
+        if( $OPTIMIZE and $self->_code_is_literal($code[0]) ) {
             $self->_fold_constants(\@code);
         }
         return @code;
@@ -1254,13 +1102,11 @@ sub _generate_call {
     my $callable = $node->first; # function or macro
     my $args     = $node->second;
 
-    if(my $intern = $builtin{$callable->id} and !$self->overridden_builtin->{$callable->id}) {
+    if(my $intern = $builtin{$callable->id}) {
         if(@{$args} != 1) {
             $self->_error("Wrong number of arguments for $callable", $node);
         }
-
-        return $self->compile_ast($args->[0]),
-            [ $intern->[0] => undef, $node->line ];
+        return $self->compile_ast($args->[0]), [ $intern => undef, $node->line ];
     }
 
     return(
@@ -1282,13 +1128,9 @@ sub _generate_iterator {
             $node);
     }
 
-    return $self->opcode(
-        load_lvar => $lvar_id + 1,
-        symbol    => $node,
-    );
+    return $self->opcode( load_lvar => $lvar_id+1, symbol => $node );
 }
 
-# $~iterator.body
 sub _generate_iterator_body {
     my($self, $node) = @_;
 
@@ -1299,10 +1141,7 @@ sub _generate_iterator_body {
             $node);
     }
 
-    return $self->opcode(
-        load_lvar => $lvar_id + 2,
-        symbol    => $node,
-    );
+    return $self->opcode( load_lvar => $lvar_id+2, symbol => $node );
 }
 
 sub _generate_assign {
@@ -1348,8 +1187,7 @@ sub _generate_constant {
     $self->{lvar_id}    = $self->lvar_use(1); # don't use local()
 
     if($OPTIMIZE) {
-        if(@expr == 1
-                && any_in($expr[0][_OP_NAME], qw(literal load_lvar))) {
+        if(@expr == 1 && any_in($expr[0][_OP_NAME], qw(literal load_lvar))) {
             $expr[0][_OP_COMMENT] = "constant $lvar_name";
             $self->const->[$lvar_id] = \@expr;
             return @expr; # no real definition
@@ -1365,17 +1203,6 @@ sub _localize_vars {
     my($self, $vars) = @_;
     my @localize;
     my @pairs = @{$vars};
-
-    if( (@pairs % 2) != 0 ) {
-        if(@pairs == 1) {
-            return $self->compile_ast(@pairs),
-                $self->opcode( 'localize_vars' );
-        }
-        else {
-            $self->_error("You must pass pairs of expressions to include");
-        }
-    }
-
     while(my($key, $expr) = splice @pairs, 0, 2) {
         if(!any_in($key->arity, qw(literal variable))) {
             $self->_error("You must pass a simple name to localize variables", $key);
@@ -1419,9 +1246,7 @@ sub can_be_in_list_context {
 
 sub _code_is_literal {
     my($self, @code) = @_;
-    return @code == 1
-        && (    $code[0][_OP_NAME] eq 'literal'
-             || $code[0][_OP_NAME] eq 'literal_i');
+    return @code == 1 && $code[0][_OP_NAME] eq 'literal';
 }
 
 sub _fold_constants {
@@ -1438,7 +1263,7 @@ sub _fold_constants {
         $engine->render('<string>');
     };
     if($@) {
-        Carp::carp("[BUG] Constant folding failed (ignored): $@");
+        Carp::carp("Oops: constant folding failed (ignored): $@");
         return 0;
     }
 
@@ -1446,6 +1271,15 @@ sub _fold_constants {
     return 1;
 }
 
+my %goto_family;
+@goto_family{qw(
+    for_iter
+    and
+    dand
+    or
+    dor
+    goto
+)} = ();
 
 sub _noop {
     my($self, $op) = @_;
@@ -1616,6 +1450,4 @@ L<Text::Xslate>
 
 L<Text::Xslate::Parser>
 
-L<Text::Xslate::Symbol>
-
 =cut
