@@ -2,7 +2,7 @@ package Text::Xslate::PP::Opcode;
 use Any::Moose;
 extends qw(Text::Xslate::PP::State);
 
-our $VERSION = '1.001';
+our $VERSION = '1.0011';
 
 use Carp ();
 use Scalar::Util ();
@@ -70,25 +70,6 @@ sub op_localize_s {
     goto $st->{ code }->[ ++$st->{ pc } ]->{ exec_code };
 }
 
-sub op_localize_vars {
-    my($st) = @_;
-    my $new_vars = $st->{sa};
-    my $old_vars = $st->vars;
-
-    if(ref($new_vars) ne 'HASH') {
-        $st->warn(undef, "Variable map must be a HASH reference");
-    }
-
-    push @{ $st->{local_stack} }, bless sub {
-            $st->vars($old_vars);
-            return;
-        }, 'Text::Xslate::PP::Guard';
-
-    $st->vars($new_vars);
-
-    goto $st->{ code }->[ ++$st->{ pc } ]->{ exec_code };
-}
-
 sub op_push {
     push @{ $_[0]->{ SP }->[ -1 ] }, $_[0]->{sa};
     goto $_[0]->{ code }->[ ++$_[0]->{ pc } ]->{ exec_code };
@@ -204,8 +185,6 @@ sub op_for_iter {
             goto $st->{ code }->[ ++$st->{ pc } ]->{ exec_code };
         }
         else {
-            # finish the loop
-            $st->{sa} = ( $i > 0 ); # for 'for-else' block
             tx_access_lvar( $st, $id + TXfor_ITEM,  undef );
             tx_access_lvar( $st, $id + TXfor_ITER,  undef );
             tx_access_lvar( $st, $id + TXfor_ARRAY, undef );
